@@ -1,3 +1,4 @@
+from django.utils.encoding import force_str
 from django.utils.six.moves.urllib import parse as urlparse
 
 
@@ -6,9 +7,9 @@ def replace_query_param(url, key, val):
     Given a URL and a key/val pair, set or replace an item in the query
     parameters of the URL, and return the new URL.
     """
-    (scheme, netloc, path, query, fragment) = urlparse.urlsplit(url)
-    query_dict = urlparse.parse_qs(query)
-    query_dict[key] = [val]
+    (scheme, netloc, path, query, fragment) = urlparse.urlsplit(force_str(url))
+    query_dict = urlparse.parse_qs(query, keep_blank_values=True)
+    query_dict[force_str(key)] = [force_str(val)]
     query = urlparse.urlencode(sorted(list(query_dict.items())), doseq=True)
     return urlparse.urlunsplit((scheme, netloc, path, query, fragment))
 
@@ -18,8 +19,8 @@ def remove_query_param(url, key):
     Given a URL and a key/val pair, remove an item in the query
     parameters of the URL, and return the new URL.
     """
-    (scheme, netloc, path, query, fragment) = urlparse.urlsplit(url)
-    query_dict = urlparse.parse_qs(query)
+    (scheme, netloc, path, query, fragment) = urlparse.urlsplit(force_str(url))
+    query_dict = urlparse.parse_qs(query, keep_blank_values=True)
     query_dict.pop(key, None)
     query = urlparse.urlencode(sorted(list(query_dict.items())), doseq=True)
     return urlparse.urlunsplit((scheme, netloc, path, query, fragment))
