@@ -24,6 +24,7 @@ from django.utils.decorators import classonlymethod
 from django.views.decorators.csrf import csrf_exempt
 
 from rest_framework import generics, mixins, views
+from rest_framework.reverse import reverse
 
 
 class ViewSetMixin(object):
@@ -46,10 +47,14 @@ class ViewSetMixin(object):
         instantiated view, we need to totally reimplement `.as_view`,
         and slightly modify the view function that is created and returned.
         """
-        # The suffix initkwarg is reserved for identifying the viewset type
+        # The suffix initkwarg is reserved for displaying the viewset type.
         # eg. 'List' or 'Instance'.
         cls.suffix = None
 
+        # Setting a basename allows a view to reverse its action urls. This
+        # value is provided by the router through the initkwargs.
+        cls.basename = None
+
         # actions must not be empty
         if not actions:
             raise TypeError("The `actions` argument must be provided when "
@@ -79,10 +84,13 @@ class ViewSetMixin(object):
                 handler = getattr(self, action)
                 setattr(self, method, handler)
 
-            # Patch this in as it's otherwise only present from 1.5 onwards
             if hasattr(self, 'get') and not hasattr(self, 'head'):
                 self.head = self.get
 
+            self.request = request
+            self.args = args
+            self.kwargs = kwargs
+
             # And continue as usual
             return self.dispatch(request, *args, **kwargs)
 
@@ -97,6 +105,7 @@ class ViewSetMixin(object):
         # generation can pick out these bits of information from a
         # resolved URL.
         view.cls = cls
+        view.initkwargs = initkwargs
         view.suffix = initkwargs.get('suffix', None)
         view.actions = actions
         return csrf_exempt(view)
@@ -111,12 +120,21 @@ class ViewSetMixin(object):
         if method == 'options':
             # This is a special case as we always provide handling for the
             # options method in the base `View` class.
-            # Unlike the other explicitly defined actions, 'metadata' is implict.
+            # Unlike the other explicitly defined actions, 'metadata' is implicit.
             self.action = 'metadata'
         else:
             self.action = self.action_map.get(method)
         return request
 
+    def reverse_action(self, url_name, *args, **kwargs):
+        """
+        Reverse the action for the given `url_name`.
+        """
+        url_name = '%s-%s' % (self.basename, url_name)
+        kwargs.setdefault('request', self.request)
+
+        return reverse(url_name, *args, **kwargs)
+
 
 class ViewSet(ViewSetMixin, views.APIView):
     """
