@@ -26,16 +26,19 @@ class DropdownWithAuthTests(TestCase):
     def test_name_shown_when_logged_in(self):
         self.client.login(username=self.username, password=self.password)
         response = self.client.get('/')
-        self.assertContains(response, 'john')
+        content = response.content.decode('utf8')
+        assert 'john' in content
 
     def test_logout_shown_when_logged_in(self):
         self.client.login(username=self.username, password=self.password)
         response = self.client.get('/')
-        self.assertContains(response, '>Log out<')
+        content = response.content.decode('utf8')
+        assert '>Log out<' in content
 
     def test_login_shown_when_logged_out(self):
         response = self.client.get('/')
-        self.assertContains(response, '>Log in<')
+        content = response.content.decode('utf8')
+        assert '>Log in<' in content
 
 
 @override_settings(ROOT_URLCONF='tests.browsable_api.no_auth_urls')
@@ -58,13 +61,16 @@ class NoDropdownWithoutAuthTests(TestCase):
     def test_name_shown_when_logged_in(self):
         self.client.login(username=self.username, password=self.password)
         response = self.client.get('/')
-        self.assertContains(response, 'john')
+        content = response.content.decode('utf8')
+        assert 'john' in content
 
     def test_dropdown_not_shown_when_logged_in(self):
         self.client.login(username=self.username, password=self.password)
         response = self.client.get('/')
-        self.assertNotContains(response, '<li class="dropdown">')
+        content = response.content.decode('utf8')
+        assert '<li class="dropdown">' not in content
 
     def test_dropdown_not_shown_when_logged_out(self):
         response = self.client.get('/')
-        self.assertNotContains(response, '<li class="dropdown">')
+        content = response.content.decode('utf8')
+        assert '<li class="dropdown">' not in content
