@@ -24,15 +24,6 @@ class BasicModel(RESTFrameworkModel):
     )
 
 
-class BaseFilterableItem(RESTFrameworkModel):
-    text = models.CharField(max_length=100)
-
-
-class FilterableItem(BaseFilterableItem):
-    decimal = models.DecimalField(max_digits=4, decimal_places=2)
-    date = models.DateField()
-
-
 # Models for relations tests
 # ManyToMany
 class ManyToManyTarget(RESTFrameworkModel):
@@ -88,3 +79,11 @@ class NullableOneToOneSource(RESTFrameworkModel):
     target = models.OneToOneField(
         OneToOneTarget, null=True, blank=True,
         related_name='nullable_source', on_delete=models.CASCADE)
+
+
+class OneToOnePKSource(RESTFrameworkModel):
+    """ Test model where the primary key is a OneToOneField with another model. """
+    name = models.CharField(max_length=100)
+    target = models.OneToOneField(
+        OneToOneTarget, primary_key=True,
+        related_name='required_source', on_delete=models.CASCADE)
