@@ -1,5 +1,6 @@
 from __future__ import unicode_literals
 
+import pytest
 from django.core.validators import MaxValueValidator, MinValueValidator
 from django.db import models
 from django.test import TestCase
@@ -8,15 +9,19 @@ from rest_framework import (
     exceptions, metadata, serializers, status, versioning, views
 )
 from rest_framework.renderers import BrowsableAPIRenderer
-from rest_framework.request import Request
 from rest_framework.test import APIRequestFactory
 
 from .models import BasicModel
 
-request = Request(APIRequestFactory().options('/'))
+request = APIRequestFactory().options('/')
 
 
 class TestMetadata:
+
+    def test_determine_metadata_abstract_method_raises_proper_error(self):
+        with pytest.raises(NotImplementedError):
+            metadata.BaseMetadata().determine_metadata(None, None)
+
     def test_metadata(self):
         """
         OPTIONS requests to views should return a valid 200 response.
@@ -263,12 +268,25 @@ class TestMetadata:
         view = ExampleView.as_view(versioning_class=scheme)
         view(request=request)
 
+    def test_list_serializer_metadata_returns_info_about_fields_of_child_serializer(self):
+        class ExampleSerializer(serializers.Serializer):
+            integer_field = serializers.IntegerField(max_value=10)
+            char_field = serializers.CharField(required=False)
+
+        class ExampleListSerializer(serializers.ListSerializer):
+            pass
+
+        options = metadata.SimpleMetadata()
+        child_serializer = ExampleSerializer()
+        list_serializer = ExampleListSerializer(child=child_serializer)
+        assert options.get_serializer_info(list_serializer) == options.get_serializer_info(child_serializer)
+
 
 class TestSimpleMetadataFieldInfo(TestCase):
     def test_null_boolean_field_info_type(self):
         options = metadata.SimpleMetadata()
         field_info = options.get_field_info(serializers.NullBooleanField())
-        self.assertEqual(field_info['type'], 'boolean')
+        assert field_info['type'] == 'boolean'
 
     def test_related_field_choices(self):
         options = metadata.SimpleMetadata()
@@ -277,7 +295,7 @@ class TestSimpleMetadataFieldInfo(TestCase):
             field_info = options.get_field_info(
                 serializers.RelatedField(queryset=BasicModel.objects.all())
             )
-        self.assertNotIn('choices', field_info)
+        assert 'choices' not in field_info
 
 
 class TestModelSerializerMetadata(TestCase):
