@@ -1,25 +1,30 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-import json
 import re
 from collections import MutableMapping, OrderedDict
 
+import pytest
 from django.conf.urls import include, url
 from django.core.cache import cache
 from django.db import models
+from django.http.request import HttpRequest
 from django.test import TestCase, override_settings
 from django.utils import six
 from django.utils.safestring import SafeText
 from django.utils.translation import ugettext_lazy as _
 
+import coreapi
 from rest_framework import permissions, serializers, status
 from rest_framework.renderers import (
-    BaseRenderer, BrowsableAPIRenderer, HTMLFormRenderer, JSONRenderer
+    AdminRenderer, BaseRenderer, BrowsableAPIRenderer, DocumentationRenderer,
+    HTMLFormRenderer, JSONRenderer, SchemaJSRenderer, StaticHTMLRenderer
 )
+from rest_framework.request import Request
 from rest_framework.response import Response
 from rest_framework.settings import api_settings
 from rest_framework.test import APIRequestFactory
+from rest_framework.utils import json
 from rest_framework.views import APIView
 
 DUMMYSTATUS = status.HTTP_200_OK
@@ -104,6 +109,7 @@ class HTMLView1(APIView):
     def get(self, request, **kwargs):
         return Response('text')
 
+
 urlpatterns = [
     url(r'^.*\.(?P<format>.+)$', MockView.as_view(renderer_classes=[RendererA, RendererB])),
     url(r'^$', MockView.as_view(renderer_classes=[RendererA, RendererB])),
@@ -238,7 +244,7 @@ class RendererEndToEndTests(TestCase):
         """
         Regression test for #1196
 
-        https://github.com/tomchristie/django-rest-framework/issues/1196
+        https://github.com/encode/django-rest-framework/issues/1196
         """
         resp = self.client.get('/empty')
         self.assertEqual(resp.get('Content-Type', None), None)
@@ -269,6 +275,18 @@ def strip_trailing_whitespace(content):
     return re.sub(' +\n', '\n', content)
 
 
+class BaseRendererTests(TestCase):
+    """
+    Tests BaseRenderer
+    """
+    def test_render_raise_error(self):
+        """
+        BaseRenderer.render should raise NotImplementedError
+        """
+        with pytest.raises(NotImplementedError):
+            BaseRenderer().render('test')
+
+
 class JSONRendererTests(TestCase):
     """
     Tests specific to the JSON Renderer
@@ -298,7 +316,7 @@ class JSONRendererTests(TestCase):
     def test_render_dict_abc_obj(self):
         class Dict(MutableMapping):
             def __init__(self):
-                self._dict = dict()
+                self._dict = {}
 
             def __getitem__(self, key):
                 return self._dict.__getitem__(key)
@@ -341,6 +359,19 @@ class JSONRendererTests(TestCase):
         with self.assertRaises(TypeError):
             JSONRenderer().render(x)
 
+    def test_float_strictness(self):
+        renderer = JSONRenderer()
+
+        # Default to strict
+        for value in [float('inf'), float('-inf'), float('nan')]:
+            with pytest.raises(ValueError):
+                renderer.render(value)
+
+        renderer.strict = False
+        assert renderer.render(float('inf')) == b'Infinity'
+        assert renderer.render(float('-inf')) == b'-Infinity'
+        assert renderer.render(float('nan')) == b'NaN'
+
     def test_without_content_type_args(self):
         """
         Test basic JSON rendering.
@@ -481,3 +512,244 @@ class TestHTMLFormRenderer(TestCase):
         result = renderer.render(self.serializer.data, None, {})
 
         self.assertIsInstance(result, SafeText)
+
+
+class TestChoiceFieldHTMLFormRenderer(TestCase):
+    """
+    Test rendering ChoiceField with HTMLFormRenderer.
+    """
+
+    def setUp(self):
+        choices = ((1, 'Option1'), (2, 'Option2'), (12, 'Option12'))
+
+        class TestSerializer(serializers.Serializer):
+            test_field = serializers.ChoiceField(choices=choices,
+                                                 initial=2)
+
+        self.TestSerializer = TestSerializer
+        self.renderer = HTMLFormRenderer()
+
+    def test_render_initial_option(self):
+        serializer = self.TestSerializer()
+        result = self.renderer.render(serializer.data)
+
+        self.assertIsInstance(result, SafeText)
+
+        self.assertInHTML('<option value="2" selected>Option2</option>',
+                          result)
+        self.assertInHTML('<option value="1">Option1</option>', result)
+        self.assertInHTML('<option value="12">Option12</option>', result)
+
+    def test_render_selected_option(self):
+        serializer = self.TestSerializer(data={'test_field': '12'})
+
+        serializer.is_valid()
+        result = self.renderer.render(serializer.data)
+
+        self.assertIsInstance(result, SafeText)
+
+        self.assertInHTML('<option value="12" selected>Option12</option>',
+                          result)
+        self.assertInHTML('<option value="1">Option1</option>', result)
+        self.assertInHTML('<option value="2">Option2</option>', result)
+
+
+class TestMultipleChoiceFieldHTMLFormRenderer(TestCase):
+    """
+    Test rendering MultipleChoiceField with HTMLFormRenderer.
+    """
+
+    def setUp(self):
+        self.renderer = HTMLFormRenderer()
+
+    def test_render_selected_option_with_string_option_ids(self):
+        choices = (('1', 'Option1'), ('2', 'Option2'), ('12', 'Option12'),
+                   ('}', 'OptionBrace'))
+
+        class TestSerializer(serializers.Serializer):
+            test_field = serializers.MultipleChoiceField(choices=choices)
+
+        serializer = TestSerializer(data={'test_field': ['12']})
+        serializer.is_valid()
+
+        result = self.renderer.render(serializer.data)
+
+        self.assertIsInstance(result, SafeText)
+
+        self.assertInHTML('<option value="12" selected>Option12</option>',
+                          result)
+        self.assertInHTML('<option value="1">Option1</option>', result)
+        self.assertInHTML('<option value="2">Option2</option>', result)
+        self.assertInHTML('<option value="}">OptionBrace</option>', result)
+
+    def test_render_selected_option_with_integer_option_ids(self):
+        choices = ((1, 'Option1'), (2, 'Option2'), (12, 'Option12'))
+
+        class TestSerializer(serializers.Serializer):
+            test_field = serializers.MultipleChoiceField(choices=choices)
+
+        serializer = TestSerializer(data={'test_field': ['12']})
+        serializer.is_valid()
+
+        result = self.renderer.render(serializer.data)
+
+        self.assertIsInstance(result, SafeText)
+
+        self.assertInHTML('<option value="12" selected>Option12</option>',
+                          result)
+        self.assertInHTML('<option value="1">Option1</option>', result)
+        self.assertInHTML('<option value="2">Option2</option>', result)
+
+
+class StaticHTMLRendererTests(TestCase):
+    """
+    Tests specific for Static HTML Renderer
+    """
+    def setUp(self):
+        self.renderer = StaticHTMLRenderer()
+
+    def test_static_renderer(self):
+        data = '<html><body>text</body></html>'
+        result = self.renderer.render(data)
+        assert result == data
+
+    def test_static_renderer_with_exception(self):
+        context = {
+            'response': Response(status=500, exception=True),
+            'request': Request(HttpRequest())
+        }
+        result = self.renderer.render({}, renderer_context=context)
+        assert result == '500 Internal Server Error'
+
+
+class BrowsableAPIRendererTests(TestCase):
+
+    def setUp(self):
+        self.renderer = BrowsableAPIRenderer()
+
+    def test_get_description_returns_empty_string_for_401_and_403_statuses(self):
+        assert self.renderer.get_description({}, status_code=401) == ''
+        assert self.renderer.get_description({}, status_code=403) == ''
+
+    def test_get_filter_form_returns_none_if_data_is_not_list_instance(self):
+        class DummyView(object):
+            get_queryset = None
+            filter_backends = None
+
+        result = self.renderer.get_filter_form(data='not list',
+                                               view=DummyView(), request={})
+        assert result is None
+
+
+class AdminRendererTests(TestCase):
+
+    def setUp(self):
+        self.renderer = AdminRenderer()
+
+    def test_render_when_resource_created(self):
+        class DummyView(APIView):
+            renderer_classes = (AdminRenderer, )
+        request = Request(HttpRequest())
+        request.build_absolute_uri = lambda: 'http://example.com'
+        response = Response(status=201, headers={'Location': '/test'})
+        context = {
+            'view': DummyView(),
+            'request': request,
+            'response': response
+        }
+
+        result = self.renderer.render(data={'test': 'test'},
+                                      renderer_context=context)
+        assert result == ''
+        assert response.status_code == status.HTTP_303_SEE_OTHER
+        assert response['Location'] == 'http://example.com'
+
+    def test_render_dict(self):
+        factory = APIRequestFactory()
+
+        class DummyView(APIView):
+            renderer_classes = (AdminRenderer, )
+
+            def get(self, request):
+                return Response({'foo': 'a string'})
+        view = DummyView.as_view()
+        request = factory.get('/')
+        response = view(request)
+        response.render()
+        self.assertContains(response, '<tr><th>Foo</th><td>a string</td></tr>', html=True)
+
+    def test_render_dict_with_items_key(self):
+        factory = APIRequestFactory()
+
+        class DummyView(APIView):
+            renderer_classes = (AdminRenderer, )
+
+            def get(self, request):
+                return Response({'items': 'a string'})
+
+        view = DummyView.as_view()
+        request = factory.get('/')
+        response = view(request)
+        response.render()
+        self.assertContains(response, '<tr><th>Items</th><td>a string</td></tr>', html=True)
+
+    def test_render_dict_with_iteritems_key(self):
+        factory = APIRequestFactory()
+
+        class DummyView(APIView):
+            renderer_classes = (AdminRenderer, )
+
+            def get(self, request):
+                return Response({'iteritems': 'a string'})
+
+        view = DummyView.as_view()
+        request = factory.get('/')
+        response = view(request)
+        response.render()
+        self.assertContains(response, '<tr><th>Iteritems</th><td>a string</td></tr>', html=True)
+
+
+class TestDocumentationRenderer(TestCase):
+
+    def test_document_with_link_named_data(self):
+        """
+        Ref #5395: Doc's `document.data` would fail with a Link named "data".
+            As per #4972, use templatetag instead.
+        """
+        document = coreapi.Document(
+            title='Data Endpoint API',
+            url='https://api.example.org/',
+            content={
+                'data': coreapi.Link(
+                    url='/data/',
+                    action='get',
+                    fields=[],
+                    description='Return data.'
+                )
+            }
+        )
+
+        factory = APIRequestFactory()
+        request = factory.get('/')
+
+        renderer = DocumentationRenderer()
+
+        html = renderer.render(document, accepted_media_type="text/html", renderer_context={"request": request})
+        assert '<h1>Data Endpoint API</h1>' in html
+
+
+class TestSchemaJSRenderer(TestCase):
+
+    def test_schemajs_output(self):
+        """
+        Test output of the SchemaJS renderer as per #5608. Django 2.0 on Py3 prints binary data as b'xyz' in templates,
+        and the base64 encoding used by SchemaJSRenderer outputs base64 as binary. Test fix.
+        """
+        factory = APIRequestFactory()
+        request = factory.get('/')
+
+        renderer = SchemaJSRenderer()
+
+        output = renderer.render('data', renderer_context={"request": request})
+        assert "'ImRhdGEi'" in output
+        assert "'b'ImRhdGEi''" not in output
