@@ -1,10 +1,15 @@
 import datetime
 
-from django.db import models
+import pytest
+from django.db import DataError, models
 from django.test import TestCase
 
 from rest_framework import serializers
-from rest_framework.validators import UniqueValidator
+from rest_framework.exceptions import ValidationError
+from rest_framework.validators import (
+    BaseUniqueForValidator, UniqueTogetherValidator, UniqueValidator,
+    qs_exists
+)
 
 
 def dedent(blocktext):
@@ -31,7 +36,7 @@ class RelatedModel(models.Model):
 
 class RelatedModelSerializer(serializers.ModelSerializer):
     username = serializers.CharField(source='user.username',
-        validators=[UniqueValidator(queryset=UniquenessModel.objects.all())])  # NOQA
+        validators=[UniqueValidator(queryset=UniquenessModel.objects.all(), lookup='iexact')])  # NOQA
 
     class Meta:
         model = RelatedModel
@@ -77,7 +82,7 @@ class TestUniquenessValidation(TestCase):
         data = {'username': 'existing'}
         serializer = UniquenessSerializer(data=data)
         assert not serializer.is_valid()
-        assert serializer.errors == {'username': ['UniquenessModel with this username already exists.']}
+        assert serializer.errors == {'username': ['uniqueness model with this username already exists.']}
 
     def test_is_unique(self):
         data = {'username': 'other'}
@@ -94,23 +99,20 @@ class TestUniquenessValidation(TestCase):
     def test_doesnt_pollute_model(self):
         instance = AnotherUniquenessModel.objects.create(code='100')
         serializer = AnotherUniquenessSerializer(instance)
-        self.assertEqual(
-            AnotherUniquenessModel._meta.get_field('code').validators, [])
+        assert AnotherUniquenessModel._meta.get_field('code').validators == []
 
         # Accessing data shouldn't effect validators on the model
         serializer.data
-        self.assertEqual(
-            AnotherUniquenessModel._meta.get_field('code').validators, [])
+        assert AnotherUniquenessModel._meta.get_field('code').validators == []
 
     def test_related_model_is_unique(self):
-        data = {'username': 'existing', 'email': 'new-email@example.com'}
+        data = {'username': 'Existing', 'email': 'new-email@example.com'}
         rs = RelatedModelSerializer(data=data)
-        self.assertFalse(rs.is_valid())
-        self.assertEqual(rs.errors,
-                         {'username': ['This field must be unique.']})
+        assert not rs.is_valid()
+        assert rs.errors == {'username': ['This field must be unique.']}
         data = {'username': 'new-username', 'email': 'new-email@example.com'}
         rs = RelatedModelSerializer(data=data)
-        self.assertTrue(rs.is_valid())
+        assert rs.is_valid()
 
     def test_value_error_treated_as_not_unique(self):
         serializer = UniquenessIntegerSerializer(data={'integer': 'abc'})
@@ -322,6 +324,23 @@ class TestUniquenessTogetherValidation(TestCase):
         serializer = NullUniquenessTogetherSerializer(data=data)
         assert not serializer.is_valid()
 
+    def test_filter_queryset_do_not_skip_existing_attribute(self):
+        """
+        filter_queryset should add value from existing instance attribute
+        if it is not provided in attributes dict
+        """
+        class MockQueryset(object):
+            def filter(self, **kwargs):
+                self.called_with = kwargs
+
+        data = {'race_name': 'bar'}
+        queryset = MockQueryset()
+        validator = UniqueTogetherValidator(queryset, fields=('race_name',
+                                                              'position'))
+        validator.instance = self.instance
+        validator.filter_queryset(attrs=data, queryset=queryset)
+        assert queryset.called_with == {'race_name': 'bar', 'position': 1}
+
 
 # Tests for `UniqueForDateValidator`
 # ----------------------------------
@@ -392,6 +411,84 @@ class TestUniquenessForDateValidation(TestCase):
             'published': datetime.date(2000, 1, 1)
         }
 
+# Tests for `UniqueForMonthValidator`
+# ----------------------------------
+
+
+class UniqueForMonthModel(models.Model):
+    slug = models.CharField(max_length=100, unique_for_month='published')
+    published = models.DateField()
+
+
+class UniqueForMonthSerializer(serializers.ModelSerializer):
+    class Meta:
+        model = UniqueForMonthModel
+        fields = '__all__'
+
+
+class UniqueForMonthTests(TestCase):
+
+    def setUp(self):
+        self.instance = UniqueForMonthModel.objects.create(
+            slug='existing', published='2017-01-01'
+        )
+
+    def test_not_unique_for_month(self):
+        data = {'slug': 'existing', 'published': '2017-01-01'}
+        serializer = UniqueForMonthSerializer(data=data)
+        assert not serializer.is_valid()
+        assert serializer.errors == {
+            'slug': ['This field must be unique for the "published" month.']
+        }
+
+    def test_unique_for_month(self):
+        data = {'slug': 'existing', 'published': '2017-02-01'}
+        serializer = UniqueForMonthSerializer(data=data)
+        assert serializer.is_valid()
+        assert serializer.validated_data == {
+            'slug': 'existing',
+            'published': datetime.date(2017, 2, 1)
+        }
+
+# Tests for `UniqueForYearValidator`
+# ----------------------------------
+
+
+class UniqueForYearModel(models.Model):
+    slug = models.CharField(max_length=100, unique_for_year='published')
+    published = models.DateField()
+
+
+class UniqueForYearSerializer(serializers.ModelSerializer):
+    class Meta:
+        model = UniqueForYearModel
+        fields = '__all__'
+
+
+class UniqueForYearTests(TestCase):
+
+    def setUp(self):
+        self.instance = UniqueForYearModel.objects.create(
+            slug='existing', published='2017-01-01'
+        )
+
+    def test_not_unique_for_year(self):
+        data = {'slug': 'existing', 'published': '2017-01-01'}
+        serializer = UniqueForYearSerializer(data=data)
+        assert not serializer.is_valid()
+        assert serializer.errors == {
+            'slug': ['This field must be unique for the "published" year.']
+        }
+
+    def test_unique_for_year(self):
+        data = {'slug': 'existing', 'published': '2018-01-01'}
+        serializer = UniqueForYearSerializer(data=data)
+        assert serializer.is_valid()
+        assert serializer.validated_data == {
+            'slug': 'existing',
+            'published': datetime.date(2018, 1, 1)
+        }
+
 
 class HiddenFieldUniqueForDateModel(models.Model):
     slug = models.CharField(max_length=100, unique_for_date='published')
@@ -432,3 +529,37 @@ class TestHiddenFieldUniquenessForDateValidation(TestCase):
                     validators = [<UniqueForDateValidator(queryset=HiddenFieldUniqueForDateModel.objects.all(), field='slug', date_field='published')>]
         """)
         assert repr(serializer) == expected
+
+
+class ValidatorsTests(TestCase):
+
+    def test_qs_exists_handles_type_error(self):
+        class TypeErrorQueryset(object):
+            def exists(self):
+                raise TypeError
+        assert qs_exists(TypeErrorQueryset()) is False
+
+    def test_qs_exists_handles_value_error(self):
+        class ValueErrorQueryset(object):
+            def exists(self):
+                raise ValueError
+        assert qs_exists(ValueErrorQueryset()) is False
+
+    def test_qs_exists_handles_data_error(self):
+        class DataErrorQueryset(object):
+            def exists(self):
+                raise DataError
+        assert qs_exists(DataErrorQueryset()) is False
+
+    def test_validator_raises_error_if_not_all_fields_are_provided(self):
+        validator = BaseUniqueForValidator(queryset=object(), field='foo',
+                                           date_field='bar')
+        attrs = {'foo': 'baz'}
+        with pytest.raises(ValidationError):
+            validator.enforce_required_fields(attrs)
+
+    def test_validator_raises_error_when_abstract_method_called(self):
+        validator = BaseUniqueForValidator(queryset=object(), field='foo',
+                                           date_field='bar')
+        with pytest.raises(NotImplementedError):
+            validator.filter_queryset(attrs=None, queryset=None)
